@@ -1,25 +1,24 @@
-from __future__ import unicode_literals
-
+import json
+import math
 import re
 from decimal import Decimal
 
 from django.contrib.gis.db.models import functions
 from django.contrib.gis.geos import (
-    LineString, Point, Polygon, fromstr, geos_version_info,
+    GEOSGeometry, LineString, Point, Polygon, fromstr,
 )
-from django.db import connection
+from django.contrib.gis.measure import Area
+from django.db import NotSupportedError, connection
+from django.db.models import Sum
 from django.test import TestCase, skipUnlessDBFeature
-from django.utils import six
 
-from ..utils import mysql, oracle, postgis, spatialite
-from .models import City, Country, State, Track
+from ..utils import FuncTestMixin, mysql, oracle, postgis, spatialite
+from .models import City, Country, CountryWebMercator, State, Track
 
 
-@skipUnlessDBFeature("gis_enabled")
-class GISFunctionsTests(TestCase):
+class GISFunctionsTests(FuncTestMixin, TestCase):
     """
     Testing functions from django/contrib/gis/db/models/functions.py.
-    Several tests are taken and adapted from GeoQuerySetTest.
     Area/Distance/Length/Perimeter are tested in distapp/tests.
 
     Please keep the tests in function's alphabetic order.
@@ -27,9 +26,8 @@ class GISFunctionsTests(TestCase):
     fixtures = ['initial']
 
     def test_asgeojson(self):
-        # Only PostGIS and SpatiaLite 3.0+ support GeoJSON.
-        if not connection.ops.geojson:
-            with self.assertRaises(NotImplementedError):
+        if not connection.features.has_AsGeoJSON_function:
+            with self.assertRaises(NotSupportedError):
                 list(Country.objects.annotate(json=functions.AsGeoJSON('mpoly')))
             return
 
@@ -46,11 +44,12 @@ class GISFunctionsTests(TestCase):
             '{"type":"Point","crs":{"type":"name","properties":{"name":"EPSG:4326"}},'
             '"bbox":[-87.65018,41.85039,-87.65018,41.85039],"coordinates":[-87.65018,41.85039]}'
         )
-        if spatialite:
-            victoria_json = (
-                '{"type":"Point","bbox":[-123.305196,48.462611,-123.305196,48.462611],'
-                '"coordinates":[-123.305196,48.462611]}'
-            )
+        # MySQL ignores the crs option.
+        if mysql:
+            houston_json = json.loads(houston_json)
+            del houston_json['crs']
+            chicago_json = json.loads(chicago_json)
+            del chicago_json['crs']
 
         # Precision argument should only be an integer
         with self.assertRaises(TypeError):
@@ -59,7 +58,7 @@ class GISFunctionsTests(TestCase):
         # Reference queries and values.
         # SELECT ST_AsGeoJson("geoapp_city"."point", 8, 0)
         # FROM "geoapp_city" WHERE "geoapp_city"."name" = 'Pueblo';
-        self.assertEqual(
+        self.assertJSONEqual(
             pueblo_json,
             City.objects.annotate(geojson=functions.AsGeoJSON('point')).get(name='Pueblo').geojson
         )
@@ -67,15 +66,15 @@ class GISFunctionsTests(TestCase):
         # SELECT ST_AsGeoJson("geoapp_city"."point", 8, 2) FROM "geoapp_city"
         # WHERE "geoapp_city"."name" = 'Houston';
         # This time we want to include the CRS by using the `crs` keyword.
-        self.assertEqual(
+        self.assertJSONEqual(
+            City.objects.annotate(json=functions.AsGeoJSON('point', crs=True)).get(name='Houston').json,
             houston_json,
-            City.objects.annotate(json=functions.AsGeoJSON('point', crs=True)).get(name='Houston').json
         )
 
         # SELECT ST_AsGeoJson("geoapp_city"."point", 8, 1) FROM "geoapp_city"
         # WHERE "geoapp_city"."name" = 'Houston';
         # This time we include the bounding box by using the `bbox` keyword.
-        self.assertEqual(
+        self.assertJSONEqual(
             victoria_json,
             City.objects.annotate(
                 geojson=functions.AsGeoJSON('point', bbox=True)
@@ -85,11 +84,14 @@ class GISFunctionsTests(TestCase):
         # SELECT ST_AsGeoJson("geoapp_city"."point", 5, 3) FROM "geoapp_city"
         # WHERE "geoapp_city"."name" = 'Chicago';
         # Finally, we set every available keyword.
-        self.assertEqual(
-            chicago_json,
+        # MariaDB doesn't limit the number of decimals in bbox.
+        if mysql and connection.mysql_is_mariadb:
+            chicago_json['bbox'] = [-87.650175, 41.850385, -87.650175, 41.850385]
+        self.assertJSONEqual(
             City.objects.annotate(
                 geojson=functions.AsGeoJSON('point', bbox=True, crs=True, precision=5)
-                ).get(name='Chicago').geojson
+            ).get(name='Chicago').geojson,
+            chicago_json,
         )
 
     @skipUnlessDBFeature("has_AsGML_function")
@@ -104,29 +106,20 @@ class GISFunctionsTests(TestCase):
         if oracle:
             # No precision parameter for Oracle :-/
             gml_regex = re.compile(
-                r'^<gml:Point srsName="SDO:4326" xmlns:gml="http://www.opengis.net/gml">'
+                r'^<gml:Point srsName="EPSG:4326" xmlns:gml="http://www.opengis.net/gml">'
                 r'<gml:coordinates decimal="\." cs="," ts=" ">-104.60925\d+,38.25500\d+ '
                 r'</gml:coordinates></gml:Point>'
             )
-        elif spatialite and connection.ops.spatial_version < (3, 0, 0):
-            # Spatialite before 3.0 has extra colon in SrsName
-            gml_regex = re.compile(
-                r'^<gml:Point SrsName="EPSG::4326"><gml:coordinates decimal="\." '
-                r'cs="," ts=" ">-104.609251\d+,38.255001</gml:coordinates></gml:Point>'
-            )
         else:
             gml_regex = re.compile(
                 r'^<gml:Point srsName="EPSG:4326"><gml:coordinates>'
                 r'-104\.60925\d+,38\.255001</gml:coordinates></gml:Point>'
             )
-
         self.assertTrue(gml_regex.match(ptown.gml))
-
-        if postgis:
-            self.assertIn(
-                '<gml:pos srsDimension="2">',
-                City.objects.annotate(gml=functions.AsGML('point', version=3)).get(name='Pueblo').gml
-            )
+        self.assertIn(
+            '<gml:pos srsDimension="2">',
+            City.objects.annotate(gml=functions.AsGML('point', version=3)).get(name='Pueblo').gml
+        )
 
     @skipUnlessDBFeature("has_AsKML_function")
     def test_askml(self):
@@ -151,15 +144,40 @@ class GISFunctionsTests(TestCase):
         self.assertEqual(svg1, City.objects.annotate(svg=functions.AsSVG('point')).get(name='Pueblo').svg)
         self.assertEqual(svg2, City.objects.annotate(svg=functions.AsSVG('point', relative=5)).get(name='Pueblo').svg)
 
+    @skipUnlessDBFeature("has_Azimuth_function")
+    def test_azimuth(self):
+        # Returns the azimuth in radians.
+        azimuth_expr = functions.Azimuth(Point(0, 0, srid=4326), Point(1, 1, srid=4326))
+        self.assertAlmostEqual(City.objects.annotate(azimuth=azimuth_expr).first().azimuth, math.pi / 4)
+        # Returns None if the two points are coincident.
+        azimuth_expr = functions.Azimuth(Point(0, 0, srid=4326), Point(0, 0, srid=4326))
+        self.assertIsNone(City.objects.annotate(azimuth=azimuth_expr).first().azimuth)
+
     @skipUnlessDBFeature("has_BoundingCircle_function")
     def test_bounding_circle(self):
+        def circle_num_points(num_seg):
+            # num_seg is the number of segments per quarter circle.
+            return (4 * num_seg) + 1
+
+        expected_areas = (169, 136) if postgis else (171, 126)
         qs = Country.objects.annotate(circle=functions.BoundingCircle('mpoly')).order_by('name')
-        self.assertAlmostEqual(qs[0].circle.area, 168.89, 2)
-        self.assertAlmostEqual(qs[1].circle.area, 135.95, 2)
+        self.assertAlmostEqual(qs[0].circle.area, expected_areas[0], 0)
+        self.assertAlmostEqual(qs[1].circle.area, expected_areas[1], 0)
+        if postgis:
+            # By default num_seg=48.
+            self.assertEqual(qs[0].circle.num_points, circle_num_points(48))
+            self.assertEqual(qs[1].circle.num_points, circle_num_points(48))
 
         qs = Country.objects.annotate(circle=functions.BoundingCircle('mpoly', num_seg=12)).order_by('name')
-        self.assertAlmostEqual(qs[0].circle.area, 168.44, 2)
-        self.assertAlmostEqual(qs[1].circle.area, 135.59, 2)
+        if postgis:
+            self.assertGreater(qs[0].circle.area, 168.4, 0)
+            self.assertLess(qs[0].circle.area, 169.5, 0)
+            self.assertAlmostEqual(qs[1].circle.area, 136, 0)
+            self.assertEqual(qs[0].circle.num_points, circle_num_points(12))
+            self.assertEqual(qs[1].circle.num_points, circle_num_points(12))
+        else:
+            self.assertAlmostEqual(qs[0].circle.area, expected_areas[0], 0)
+            self.assertAlmostEqual(qs[1].circle.area, expected_areas[1], 0)
 
     @skipUnlessDBFeature("has_Centroid_function")
     def test_centroid(self):
@@ -168,24 +186,27 @@ class GISFunctionsTests(TestCase):
         for state in qs:
             self.assertTrue(state.poly.centroid.equals_exact(state.centroid, tol))
 
+        with self.assertRaisesMessage(TypeError, "'Centroid' takes exactly 1 argument (2 given)"):
+            State.objects.annotate(centroid=functions.Centroid('poly', 'poly'))
+
     @skipUnlessDBFeature("has_Difference_function")
     def test_difference(self):
         geom = Point(5, 23, srid=4326)
         qs = Country.objects.annotate(diff=functions.Difference('mpoly', geom))
-        # SpatiaLite and Oracle do something screwy with the Texas geometry.
-        if spatialite or oracle:
+        # Oracle does something screwy with the Texas geometry.
+        if oracle:
             qs = qs.exclude(name='Texas')
 
         for c in qs:
             self.assertTrue(c.mpoly.difference(geom).equals(c.diff))
 
-    @skipUnlessDBFeature("has_Difference_function")
+    @skipUnlessDBFeature("has_Difference_function", "has_Transform_function")
     def test_difference_mixed_srid(self):
         """Testing with mixed SRID (Country has default 4326)."""
-        geom = Point(556597.4, 2632018.6, srid=3857)  # Spherical mercator
+        geom = Point(556597.4, 2632018.6, srid=3857)  # Spherical Mercator
         qs = Country.objects.annotate(difference=functions.Difference('mpoly', geom))
-        # SpatiaLite and Oracle do something screwy with the Texas geometry.
-        if spatialite or oracle:
+        # Oracle does something screwy with the Texas geometry.
+        if oracle:
             qs = qs.exclude(name='Texas')
         for c in qs:
             self.assertTrue(c.mpoly.difference(geom).equals(c.difference))
@@ -194,10 +215,10 @@ class GISFunctionsTests(TestCase):
     def test_envelope(self):
         countries = Country.objects.annotate(envelope=functions.Envelope('mpoly'))
         for country in countries:
-            self.assertIsInstance(country.envelope, Polygon)
+            self.assertTrue(country.envelope.equals(country.mpoly.envelope))
 
-    @skipUnlessDBFeature("has_ForceRHR_function")
-    def test_force_rhr(self):
+    @skipUnlessDBFeature("has_ForcePolygonCW_function")
+    def test_force_polygon_cw(self):
         rings = (
             ((0, 0), (5, 0), (0, 5), (0, 0)),
             ((1, 1), (1, 3), (3, 1), (1, 1)),
@@ -207,8 +228,8 @@ class GISFunctionsTests(TestCase):
             ((1, 1), (3, 1), (1, 3), (1, 1)),
         )
         State.objects.create(name='Foo', poly=Polygon(*rings))
-        st = State.objects.annotate(force_rhr=functions.ForceRHR('poly')).get(name='Foo')
-        self.assertEqual(rhr_rings, st.force_rhr.coords)
+        st = State.objects.annotate(force_polygon_cw=functions.ForcePolygonCW('poly')).get(name='Foo')
+        self.assertEqual(rhr_rings, st.force_polygon_cw.coords)
 
     @skipUnlessDBFeature("has_GeoHash_function")
     def test_geohash(self):
@@ -218,24 +239,92 @@ class GISFunctionsTests(TestCase):
         ref_hash = '9vk1mfq8jx0c8e0386z6'
         h1 = City.objects.annotate(geohash=functions.GeoHash('point')).get(name='Houston')
         h2 = City.objects.annotate(geohash=functions.GeoHash('point', precision=5)).get(name='Houston')
-        self.assertEqual(ref_hash, h1.geohash)
+        self.assertEqual(ref_hash, h1.geohash[:len(ref_hash)])
         self.assertEqual(ref_hash[:5], h2.geohash)
 
+    @skipUnlessDBFeature('has_GeometryDistance_function')
+    def test_geometry_distance(self):
+        point = Point(-90, 40, srid=4326)
+        qs = City.objects.annotate(distance=functions.GeometryDistance('point', point)).order_by('distance')
+        distances = (
+            2.99091995527296,
+            5.33507274054713,
+            9.33852187483721,
+            9.91769193646233,
+            11.556465744884,
+            14.713098433352,
+            34.3635252198568,
+            276.987855073372,
+        )
+        for city, expected_distance in zip(qs, distances):
+            with self.subTest(city=city):
+                self.assertAlmostEqual(city.distance, expected_distance)
+
     @skipUnlessDBFeature("has_Intersection_function")
     def test_intersection(self):
         geom = Point(5, 23, srid=4326)
         qs = Country.objects.annotate(inter=functions.Intersection('mpoly', geom))
         for c in qs:
-            if spatialite:
-                # When the intersection is empty, Spatialite returns None
+            if spatialite or (mysql and not connection.features.supports_empty_geometry_collection) or oracle:
+                # When the intersection is empty, some databases return None.
                 expected = None
-            elif oracle:
-                # When the intersection is empty, Oracle returns an empty string
-                expected = ''
             else:
                 expected = c.mpoly.intersection(geom)
             self.assertEqual(c.inter, expected)
 
+    @skipUnlessDBFeature("has_IsValid_function")
+    def test_isvalid(self):
+        valid_geom = fromstr('POLYGON((0 0, 0 1, 1 1, 1 0, 0 0))')
+        invalid_geom = fromstr('POLYGON((0 0, 0 1, 1 1, 1 0, 1 1, 1 0, 0 0))')
+        State.objects.create(name='valid', poly=valid_geom)
+        State.objects.create(name='invalid', poly=invalid_geom)
+        valid = State.objects.filter(name='valid').annotate(isvalid=functions.IsValid('poly')).first()
+        invalid = State.objects.filter(name='invalid').annotate(isvalid=functions.IsValid('poly')).first()
+        self.assertIs(valid.isvalid, True)
+        self.assertIs(invalid.isvalid, False)
+
+    @skipUnlessDBFeature("has_Area_function")
+    def test_area_with_regular_aggregate(self):
+        # Create projected country objects, for this test to work on all backends.
+        for c in Country.objects.all():
+            CountryWebMercator.objects.create(name=c.name, mpoly=c.mpoly.transform(3857, clone=True))
+        # Test in projected coordinate system
+        qs = CountryWebMercator.objects.annotate(area_sum=Sum(functions.Area('mpoly')))
+        # Some backends (e.g. Oracle) cannot group by multipolygon values, so
+        # defer such fields in the aggregation query.
+        for c in qs.defer('mpoly'):
+            result = c.area_sum
+            # If the result is a measure object, get value.
+            if isinstance(result, Area):
+                result = result.sq_m
+            self.assertAlmostEqual((result - c.mpoly.area) / c.mpoly.area, 0)
+
+    @skipUnlessDBFeature("has_Area_function")
+    def test_area_lookups(self):
+        # Create projected countries so the test works on all backends.
+        CountryWebMercator.objects.bulk_create(
+            CountryWebMercator(name=c.name, mpoly=c.mpoly.transform(3857, clone=True))
+            for c in Country.objects.all()
+        )
+        qs = CountryWebMercator.objects.annotate(area=functions.Area('mpoly'))
+        self.assertEqual(qs.get(area__lt=Area(sq_km=500000)), CountryWebMercator.objects.get(name='New Zealand'))
+
+        with self.assertRaisesMessage(ValueError, 'AreaField only accepts Area measurement objects.'):
+            qs.get(area__lt=500000)
+
+    @skipUnlessDBFeature("has_LineLocatePoint_function")
+    def test_line_locate_point(self):
+        pos_expr = functions.LineLocatePoint(LineString((0, 0), (0, 3), srid=4326), Point(0, 1, srid=4326))
+        self.assertAlmostEqual(State.objects.annotate(pos=pos_expr).first().pos, 0.3333333)
+
+    @skipUnlessDBFeature("has_MakeValid_function")
+    def test_make_valid(self):
+        invalid_geom = fromstr('POLYGON((0 0, 0 1, 1 1, 1 0, 1 1, 1 0, 0 0))')
+        State.objects.create(name='invalid', poly=invalid_geom)
+        invalid = State.objects.filter(name='invalid').annotate(repaired=functions.MakeValid('poly')).first()
+        self.assertIs(invalid.repaired.valid, True)
+        self.assertEqual(invalid.repaired, fromstr('POLYGON((0 0, 0 1, 1 1, 1 0, 0 0))', srid=invalid.poly.srid))
+
     @skipUnlessDBFeature("has_MemSize_function")
     def test_memsize(self):
         ptown = City.objects.annotate(size=functions.MemSize('point')).get(name='Pueblo')
@@ -262,38 +351,23 @@ class GISFunctionsTests(TestCase):
         Track.objects.create(name='Foo', line=LineString(coords))
         qs = Track.objects.annotate(num_points=functions.NumPoints('line'))
         self.assertEqual(qs.first().num_points, 2)
-        if spatialite or mysql:
-            # Spatialite and MySQL can only count points on LineStrings
+        mpoly_qs = Country.objects.annotate(num_points=functions.NumPoints('mpoly'))
+        if not connection.features.supports_num_points_poly:
+            for c in mpoly_qs:
+                self.assertIsNone(c.num_points)
             return
 
-        for c in Country.objects.annotate(num_points=functions.NumPoints('mpoly')):
+        for c in mpoly_qs:
             self.assertEqual(c.mpoly.num_points, c.num_points)
 
-        if not oracle:
-            # Oracle cannot count vertices in Point geometries.
-            for c in City.objects.annotate(num_points=functions.NumPoints('point')):
-                self.assertEqual(1, c.num_points)
+        for c in City.objects.annotate(num_points=functions.NumPoints('point')):
+            self.assertEqual(c.num_points, 1)
 
     @skipUnlessDBFeature("has_PointOnSurface_function")
     def test_point_on_surface(self):
-        # Reference values.
-        if oracle:
-            # SELECT SDO_UTIL.TO_WKTGEOMETRY(SDO_GEOM.SDO_POINTONSURFACE(GEOAPP_COUNTRY.MPOLY, 0.05))
-            # FROM GEOAPP_COUNTRY;
-            ref = {'New Zealand': fromstr('POINT (174.616364 -36.100861)', srid=4326),
-                   'Texas': fromstr('POINT (-103.002434 36.500397)', srid=4326),
-                   }
-        else:
-            # Using GEOSGeometry to compute the reference point on surface values
-            # -- since PostGIS also uses GEOS these should be the same.
-            ref = {'New Zealand': Country.objects.get(name='New Zealand').mpoly.point_on_surface,
-                   'Texas': Country.objects.get(name='Texas').mpoly.point_on_surface
-                   }
-
         qs = Country.objects.annotate(point_on_surface=functions.PointOnSurface('mpoly'))
         for country in qs:
-            tol = 0.00001  # Spatialite might have WKT-translation-related precision issues
-            self.assertTrue(ref[country.name].equals_exact(country.point_on_surface, tol))
+            self.assertTrue(country.mpoly.intersection(country.point_on_surface))
 
     @skipUnlessDBFeature("has_Reverse_function")
     def test_reverse_geom(self):
@@ -324,7 +398,7 @@ class GISFunctionsTests(TestCase):
         for bad_args in ((), range(3), range(5)):
             with self.assertRaises(ValueError):
                 Country.objects.annotate(snap=functions.SnapToGrid('mpoly', *bad_args))
-        for bad_args in (('1.0',), (1.0, None), tuple(map(six.text_type, range(4)))):
+        for bad_args in (('1.0',), (1.0, None), tuple(map(str, range(4)))):
             with self.assertRaises(TypeError):
                 Country.objects.annotate(snap=functions.SnapToGrid('mpoly', *bad_args))
 
@@ -384,17 +458,13 @@ class GISFunctionsTests(TestCase):
 
     @skipUnlessDBFeature("has_SymDifference_function")
     def test_sym_difference(self):
-        if geos_version_info()['version'] < '3.3.0':
-            self.skipTest("GEOS >= 3.3 required")
         geom = Point(5, 23, srid=4326)
         qs = Country.objects.annotate(sym_difference=functions.SymDifference('mpoly', geom))
         # Oracle does something screwy with the Texas geometry.
         if oracle:
             qs = qs.exclude(name='Texas')
         for country in qs:
-            # Ordering might differ in collections
-            self.assertSetEqual(set(g.wkt for g in country.mpoly.sym_difference(geom)),
-                                set(g.wkt for g in country.sym_difference))
+            self.assertTrue(country.mpoly.sym_difference(geom).equals(country.sym_difference))
 
     @skipUnlessDBFeature("has_Transform_function")
     def test_transform(self):
@@ -426,42 +496,70 @@ class GISFunctionsTests(TestCase):
         "has_Difference_function", "has_Intersection_function",
         "has_SymDifference_function", "has_Union_function")
     def test_diff_intersection_union(self):
-        "Testing the `difference`, `intersection`, `sym_difference`, and `union` GeoQuerySet methods."
         geom = Point(5, 23, srid=4326)
         qs = Country.objects.all().annotate(
             difference=functions.Difference('mpoly', geom),
             sym_difference=functions.SymDifference('mpoly', geom),
             union=functions.Union('mpoly', geom),
+            intersection=functions.Intersection('mpoly', geom),
         )
 
-        # For some reason SpatiaLite does something screwy with the Texas geometry here.
-        # Also, it doesn't like the null intersection.
-        if spatialite:
-            qs = qs.exclude(name='Texas')
-        else:
-            qs = qs.annotate(intersection=functions.Intersection('mpoly', geom))
-
         if oracle:
             # Should be able to execute the queries; however, they won't be the same
             # as GEOS (because Oracle doesn't use GEOS internally like PostGIS or
             # SpatiaLite).
             return
         for c in qs:
-            self.assertEqual(c.mpoly.difference(geom), c.difference)
-            if not spatialite:
+            self.assertTrue(c.mpoly.difference(geom).equals(c.difference))
+            if not (spatialite or mysql):
                 self.assertEqual(c.mpoly.intersection(geom), c.intersection)
-            # Ordering might differ in collections
-            self.assertSetEqual(set(g.wkt for g in c.mpoly.sym_difference(geom)),
-                                set(g.wkt for g in c.sym_difference))
-            self.assertSetEqual(set(g.wkt for g in c.mpoly.union(geom)),
-                                set(g.wkt for g in c.union))
+            self.assertTrue(c.mpoly.sym_difference(geom).equals(c.sym_difference))
+            self.assertTrue(c.mpoly.union(geom).equals(c.union))
 
     @skipUnlessDBFeature("has_Union_function")
     def test_union(self):
+        """Union with all combinations of geometries/geometry fields."""
         geom = Point(-95.363151, 29.763374, srid=4326)
-        ptown = City.objects.annotate(union=functions.Union('point', geom)).get(name='Dallas')
-        tol = 0.00001
-        # Undefined ordering
-        expected1 = fromstr('MULTIPOINT(-96.801611 32.782057,-95.363151 29.763374)', srid=4326)
-        expected2 = fromstr('MULTIPOINT(-95.363151 29.763374,-96.801611 32.782057)', srid=4326)
-        self.assertTrue(expected1.equals_exact(ptown.union, tol) or expected2.equals_exact(ptown.union, tol))
+
+        union = City.objects.annotate(union=functions.Union('point', geom)).get(name='Dallas').union
+        expected = fromstr('MULTIPOINT(-96.801611 32.782057,-95.363151 29.763374)', srid=4326)
+        self.assertTrue(expected.equals(union))
+
+        union = City.objects.annotate(union=functions.Union(geom, 'point')).get(name='Dallas').union
+        self.assertTrue(expected.equals(union))
+
+        union = City.objects.annotate(union=functions.Union('point', 'point')).get(name='Dallas').union
+        expected = GEOSGeometry('POINT(-96.801611 32.782057)', srid=4326)
+        self.assertTrue(expected.equals(union))
+
+        union = City.objects.annotate(union=functions.Union(geom, geom)).get(name='Dallas').union
+        self.assertTrue(geom.equals(union))
+
+    @skipUnlessDBFeature("has_Union_function", "has_Transform_function")
+    def test_union_mixed_srid(self):
+        """The result SRID depends on the order of parameters."""
+        geom = Point(61.42915, 55.15402, srid=4326)
+        geom_3857 = geom.transform(3857, clone=True)
+        tol = 0.001
+
+        for city in City.objects.annotate(union=functions.Union('point', geom_3857)):
+            expected = city.point | geom
+            self.assertTrue(city.union.equals_exact(expected, tol))
+            self.assertEqual(city.union.srid, 4326)
+
+        for city in City.objects.annotate(union=functions.Union(geom_3857, 'point')):
+            expected = geom_3857 | city.point.transform(3857, clone=True)
+            self.assertTrue(expected.equals_exact(city.union, tol))
+            self.assertEqual(city.union.srid, 3857)
+
+    def test_argument_validation(self):
+        with self.assertRaisesMessage(ValueError, 'SRID is required for all geometries.'):
+            City.objects.annotate(geo=functions.GeoFunc(Point(1, 1)))
+
+        msg = 'GeoFunc function requires a GeometryField in position 1, got CharField.'
+        with self.assertRaisesMessage(TypeError, msg):
+            City.objects.annotate(geo=functions.GeoFunc('name'))
+
+        msg = 'GeoFunc function requires a geometric argument in position 1.'
+        with self.assertRaisesMessage(TypeError, msg):
+            City.objects.annotate(union=functions.GeoFunc(1, 'point')).get(name='Dallas')
