@@ -1,92 +1,50 @@
-# -*- encoding: utf-8 -*-
-from __future__ import unicode_literals
-
-import io
 import os
 import re
 import shutil
+import tempfile
 import time
 import warnings
-from unittest import SkipTest, skipUnless
+from io import StringIO
+from unittest import mock, skipIf, skipUnless
+
+from admin_scripts.tests import AdminScriptTestCase
 
-from django.conf import settings
 from django.core import management
 from django.core.management import execute_from_command_line
 from django.core.management.base import CommandError
-from django.core.management.commands.makemessages import \
-    Command as MakeMessagesCommand
+from django.core.management.commands.makemessages import (
+    Command as MakeMessagesCommand, write_pot_file,
+)
 from django.core.management.utils import find_command
-from django.test import SimpleTestCase, mock, override_settings
-from django.test.testcases import SerializeMixin
+from django.test import SimpleTestCase, override_settings
 from django.test.utils import captured_stderr, captured_stdout
-from django.utils import six
-from django.utils._os import upath
-from django.utils.encoding import force_text
-from django.utils.six import StringIO
+from django.utils._os import symlinks_supported
 from django.utils.translation import TranslatorCommentWarning
 
+from .utils import POFileAssertionMixin, RunInTmpDirMixin, copytree
+
 LOCALE = 'de'
 has_xgettext = find_command('xgettext')
-this_directory = os.path.dirname(upath(__file__))
+gettext_version = MakeMessagesCommand().gettext_version if has_xgettext else None
+requires_gettext_019 = skipIf(has_xgettext and gettext_version < (0, 19), 'gettext 0.19 required')
 
 
 @skipUnless(has_xgettext, 'xgettext is mandatory for extraction tests')
-class ExtractorTests(SerializeMixin, SimpleTestCase):
+class ExtractorTests(POFileAssertionMixin, RunInTmpDirMixin, SimpleTestCase):
 
-    # makemessages scans the current working directory and writes in the
-    # locale subdirectory. There aren't any options to control this. As a
-    # consequence tests can't run in parallel. Since i18n tests run in less
-    # than 4 seconds, serializing them with SerializeMixin is acceptable.
-    lockfile = __file__
-
-    test_dir = os.path.abspath(os.path.join(this_directory, 'commands'))
+    work_subdir = 'commands'
 
     PO_FILE = 'locale/%s/LC_MESSAGES/django.po' % LOCALE
 
-    def setUp(self):
-        self._cwd = os.getcwd()
-
-    def _rmrf(self, dname):
-        if os.path.commonprefix([self.test_dir, os.path.abspath(dname)]) != self.test_dir:
-            return
-        shutil.rmtree(dname)
-
-    def rmfile(self, filepath):
-        if os.path.exists(filepath):
-            os.remove(filepath)
-
-    def tearDown(self):
-        os.chdir(self.test_dir)
-        try:
-            self._rmrf('locale/%s' % LOCALE)
-        except OSError:
-            pass
-        os.chdir(self._cwd)
-
     def _run_makemessages(self, **options):
-        os.chdir(self.test_dir)
         out = StringIO()
-        management.call_command('makemessages', locale=[LOCALE], verbosity=2,
-            stdout=out, **options)
+        management.call_command('makemessages', locale=[LOCALE], verbosity=2, stdout=out, **options)
         output = out.getvalue()
         self.assertTrue(os.path.exists(self.PO_FILE))
-        with open(self.PO_FILE, 'r') as fp:
+        with open(self.PO_FILE) as fp:
             po_contents = fp.read()
         return output, po_contents
 
-    def _assertPoKeyword(self, keyword, expected_value, haystack, use_quotes=True):
-        q = '"'
-        if use_quotes:
-            expected_value = '"%s"' % expected_value
-            q = "'"
-        needle = '%s %s' % (keyword, expected_value)
-        expected_value = re.escape(expected_value)
-        return self.assertTrue(re.search('^%s %s' % (keyword, expected_value), haystack, re.MULTILINE),
-                               'Could not find %(q)s%(n)s%(q)s in generated PO file' % {'n': needle, 'q': q})
-
-    def assertMsgId(self, msgid, haystack, use_quotes=True):
-        return self._assertPoKeyword('msgid', msgid, haystack, use_quotes=use_quotes)
-
     def assertMsgIdPlural(self, msgid, haystack, use_quotes=True):
         return self._assertPoKeyword('msgid_plural', msgid, haystack, use_quotes=use_quotes)
 
@@ -100,39 +58,39 @@ class ExtractorTests(SerializeMixin, SimpleTestCase):
         return self.assertTrue(not re.search('^msgid %s' % msgid, s, re.MULTILINE))
 
     def _assertPoLocComment(self, assert_presence, po_filename, line_number, *comment_parts):
-        with open(po_filename, 'r') as fp:
-            po_contents = force_text(fp.read())
+        with open(po_filename) as fp:
+            po_contents = fp.read()
         if os.name == 'nt':
             # #: .\path\to\file.html:123
             cwd_prefix = '%s%s' % (os.curdir, os.sep)
         else:
             # #: path/to/file.html:123
             cwd_prefix = ''
-        parts = ['#: ']
 
         path = os.path.join(cwd_prefix, *comment_parts)
-        parts.append(path)
+        parts = [path]
 
-        if isinstance(line_number, six.string_types):
+        if isinstance(line_number, str):
             line_number = self._get_token_line_number(path, line_number)
         if line_number is not None:
             parts.append(':%d' % line_number)
 
         needle = ''.join(parts)
+        pattern = re.compile(r'^\#\:.*' + re.escape(needle), re.MULTILINE)
         if assert_presence:
-            return self.assertIn(needle, po_contents, '"%s" not found in final .po file.' % needle)
+            return self.assertRegex(po_contents, pattern, '"%s" not found in final .po file.' % needle)
         else:
-            return self.assertNotIn(needle, po_contents, '"%s" shouldn\'t be in final .po file.' % needle)
+            return self.assertNotRegex(po_contents, pattern, '"%s" shouldn\'t be in final .po file.' % needle)
 
     def _get_token_line_number(self, path, token):
         with open(path) as f:
             for line, content in enumerate(f, 1):
-                if token in force_text(content):
+                if token in content:
                     return line
         self.fail("The token '%s' could not be found in %s, please check the test config" % (token, path))
 
     def assertLocationCommentPresent(self, po_filename, line_number, *comment_parts):
-        """
+        r"""
         self.assertLocationCommentPresent('django.po', 42, 'dirA', 'dirB', 'foo.py')
 
         verifies that the django.po file has a gettext-style location comment of the form
@@ -170,11 +128,23 @@ class ExtractorTests(SerializeMixin, SimpleTestCase):
 
 class BasicExtractorTests(ExtractorTests):
 
-    def test_comments_extractor(self):
-        os.chdir(self.test_dir)
+    @override_settings(USE_I18N=False)
+    def test_use_i18n_false(self):
+        """
+        makemessages also runs successfully when USE_I18N is False.
+        """
         management.call_command('makemessages', locale=[LOCALE], verbosity=0)
         self.assertTrue(os.path.exists(self.PO_FILE))
-        with io.open(self.PO_FILE, 'r', encoding='utf-8') as fp:
+        with open(self.PO_FILE, encoding='utf-8') as fp:
+            po_contents = fp.read()
+            # Check two random strings
+            self.assertIn('#. Translators: One-line translator comment #1', po_contents)
+            self.assertIn('msgctxt "Special trans context #1"', po_contents)
+
+    def test_comments_extractor(self):
+        management.call_command('makemessages', locale=[LOCALE], verbosity=0)
+        self.assertTrue(os.path.exists(self.PO_FILE))
+        with open(self.PO_FILE, encoding='utf-8') as fp:
             po_contents = fp.read()
             self.assertNotIn('This comment should not be extracted', po_contents)
 
@@ -204,12 +174,18 @@ class BasicExtractorTests(ExtractorTests):
                 po_contents
             )
 
-    def test_blocktrans_trimmed(self):
-        os.chdir(self.test_dir)
+    def test_special_char_extracted(self):
         management.call_command('makemessages', locale=[LOCALE], verbosity=0)
         self.assertTrue(os.path.exists(self.PO_FILE))
-        with open(self.PO_FILE, 'r') as fp:
-            po_contents = force_text(fp.read())
+        with open(self.PO_FILE, encoding='utf-8') as fp:
+            po_contents = fp.read()
+            self.assertMsgId("Non-breaking space\u00a0:", po_contents)
+
+    def test_blocktrans_trimmed(self):
+        management.call_command('makemessages', locale=[LOCALE], verbosity=0)
+        self.assertTrue(os.path.exists(self.PO_FILE))
+        with open(self.PO_FILE) as fp:
+            po_contents = fp.read()
             # should not be trimmed
             self.assertNotMsgId('Text with a few line breaks.', po_contents)
             # should be trimmed
@@ -218,50 +194,42 @@ class BasicExtractorTests(ExtractorTests):
         self.assertMsgId("Get my line number", po_contents)
         self.assertLocationCommentPresent(self.PO_FILE, 'Get my line number', 'templates', 'test.html')
 
-    def test_force_en_us_locale(self):
-        """Value of locale-munging option used by the command is the right one"""
-        self.assertTrue(MakeMessagesCommand.leave_locale_alone)
-
     def test_extraction_error(self):
-        os.chdir(self.test_dir)
         msg = (
             'Translation blocks must not include other block tags: blocktrans '
             '(file %s, line 3)' % os.path.join('templates', 'template_with_error.tpl')
         )
         with self.assertRaisesMessage(SyntaxError, msg):
             management.call_command('makemessages', locale=[LOCALE], extensions=['tpl'], verbosity=0)
-        # Check that the temporary file was cleaned up
+        # The temporary file was cleaned up
         self.assertFalse(os.path.exists('./templates/template_with_error.tpl.py'))
 
     def test_unicode_decode_error(self):
-        os.chdir(self.test_dir)
         shutil.copyfile('./not_utf8.sample', './not_utf8.txt')
-        self.addCleanup(self.rmfile, os.path.join(self.test_dir, 'not_utf8.txt'))
         out = StringIO()
         management.call_command('makemessages', locale=[LOCALE], stdout=out)
-        self.assertIn("UnicodeDecodeError: skipped file not_utf8.txt in .",
-                      force_text(out.getvalue()))
+        self.assertIn("UnicodeDecodeError: skipped file not_utf8.txt in .", out.getvalue())
+
+    def test_unicode_file_name(self):
+        open(os.path.join(self.test_dir, 'vidéo.txt'), 'a').close()
+        management.call_command('makemessages', locale=[LOCALE], verbosity=0)
 
     def test_extraction_warning(self):
         """test xgettext warning about multiple bare interpolation placeholders"""
-        os.chdir(self.test_dir)
         shutil.copyfile('./code.sample', './code_sample.py')
-        self.addCleanup(self.rmfile, os.path.join(self.test_dir, 'code_sample.py'))
         out = StringIO()
         management.call_command('makemessages', locale=[LOCALE], stdout=out)
-        self.assertIn("code_sample.py:4", force_text(out.getvalue()))
+        self.assertIn("code_sample.py:4", out.getvalue())
 
     def test_template_message_context_extractor(self):
         """
-        Ensure that message contexts are correctly extracted for the
-        {% trans %} and {% blocktrans %} template tags.
-        Refs #14806.
+        Message contexts are correctly extracted for the {% trans %} and
+        {% blocktrans %} template tags (#14806).
         """
-        os.chdir(self.test_dir)
         management.call_command('makemessages', locale=[LOCALE], verbosity=0)
         self.assertTrue(os.path.exists(self.PO_FILE))
-        with open(self.PO_FILE, 'r') as fp:
-            po_contents = force_text(fp.read())
+        with open(self.PO_FILE) as fp:
+            po_contents = fp.read()
             # {% trans %}
             self.assertIn('msgctxt "Special trans context #1"', po_contents)
             self.assertMsgId("Translatable literal #7a", po_contents)
@@ -288,11 +256,10 @@ class BasicExtractorTests(ExtractorTests):
             self.assertMsgId("Translatable literal #8d %(a)s", po_contents)
 
     def test_context_in_single_quotes(self):
-        os.chdir(self.test_dir)
         management.call_command('makemessages', locale=[LOCALE], verbosity=0)
         self.assertTrue(os.path.exists(self.PO_FILE))
-        with open(self.PO_FILE, 'r') as fp:
-            po_contents = force_text(fp.read())
+        with open(self.PO_FILE) as fp:
+            po_contents = fp.read()
             # {% trans %}
             self.assertIn('msgctxt "Context wrapped in double quotes"', po_contents)
             self.assertIn('msgctxt "Context wrapped in single quotes"', po_contents)
@@ -303,7 +270,6 @@ class BasicExtractorTests(ExtractorTests):
 
     def test_template_comments(self):
         """Template comment tags on the same line of other constructs (#19552)"""
-        os.chdir(self.test_dir)
         # Test detection/end user reporting of old, incorrect templates
         # translator comments syntax
         with warnings.catch_warnings(record=True) as ws:
@@ -312,28 +278,28 @@ class BasicExtractorTests(ExtractorTests):
             self.assertEqual(len(ws), 3)
             for w in ws:
                 self.assertTrue(issubclass(w.category, TranslatorCommentWarning))
-            six.assertRegex(
-                self, str(ws[0].message),
+            self.assertRegex(
+                str(ws[0].message),
                 r"The translator-targeted comment 'Translators: ignored i18n "
                 r"comment #1' \(file templates[/\\]comments.thtml, line 4\) "
                 r"was ignored, because it wasn't the last item on the line\."
             )
-            six.assertRegex(
-                self, str(ws[1].message),
+            self.assertRegex(
+                str(ws[1].message),
                 r"The translator-targeted comment 'Translators: ignored i18n "
                 r"comment #3' \(file templates[/\\]comments.thtml, line 6\) "
                 r"was ignored, because it wasn't the last item on the line\."
             )
-            six.assertRegex(
-                self, str(ws[2].message),
+            self.assertRegex(
+                str(ws[2].message),
                 r"The translator-targeted comment 'Translators: ignored i18n "
                 r"comment #4' \(file templates[/\\]comments.thtml, line 8\) "
-                "was ignored, because it wasn't the last item on the line\."
+                r"was ignored, because it wasn't the last item on the line\."
             )
         # Now test .po file contents
         self.assertTrue(os.path.exists(self.PO_FILE))
-        with open(self.PO_FILE, 'r') as fp:
-            po_contents = force_text(fp.read())
+        with open(self.PO_FILE) as fp:
+            po_contents = fp.read()
 
             self.assertMsgId('Translatable literal #9a', po_contents)
             self.assertNotIn('ignored comment #1', po_contents)
@@ -363,13 +329,13 @@ class BasicExtractorTests(ExtractorTests):
             self.assertIn('#. Translators: valid i18n comment #7', po_contents)
             self.assertMsgId('Translatable literal #9i', po_contents)
 
-            six.assertRegex(self, po_contents, r'#\..+Translators: valid i18n comment #8')
-            six.assertRegex(self, po_contents, r'#\..+Translators: valid i18n comment #9')
+            self.assertRegex(po_contents, r'#\..+Translators: valid i18n comment #8')
+            self.assertRegex(po_contents, r'#\..+Translators: valid i18n comment #9')
             self.assertMsgId("Translatable literal #9j", po_contents)
 
     def test_makemessages_find_files(self):
         """
-        Test that find_files only discover files having the proper extensions.
+        find_files only discover files having the proper extensions.
         """
         cmd = MakeMessagesCommand()
         cmd.ignore_patterns = ['CVS', '.*', '*~', '*.pyc']
@@ -380,13 +346,13 @@ class BasicExtractorTests(ExtractorTests):
         cmd.locale_paths = []
         cmd.default_locale_path = os.path.join(self.test_dir, 'locale')
         found_files = cmd.find_files(self.test_dir)
-        found_exts = set([os.path.splitext(tfile.file)[1] for tfile in found_files])
+        found_exts = {os.path.splitext(tfile.file)[1] for tfile in found_files}
         self.assertEqual(found_exts.difference({'.py', '.html', '.txt'}), set())
 
         cmd.extensions = ['js']
         cmd.domain = 'djangojs'
         found_files = cmd.find_files(self.test_dir)
-        found_exts = set([os.path.splitext(tfile.file)[1] for tfile in found_files])
+        found_exts = {os.path.splitext(tfile.file)[1] for tfile in found_files}
         self.assertEqual(found_exts.difference({'.js'}), set())
 
     @mock.patch('django.core.management.commands.makemessages.popen_wrapper')
@@ -412,28 +378,48 @@ class BasicExtractorTests(ExtractorTests):
         mocked_popen_wrapper.return_value = (
             "any other return value\n", '', 0)
         cmd = MakeMessagesCommand()
-        with six.assertRaisesRegex(self, CommandError, "Unable to get gettext version. Is it installed?"):
+        with self.assertRaisesMessage(CommandError, "Unable to get gettext version. Is it installed?"):
             cmd.gettext_version
 
     def test_po_file_encoding_when_updating(self):
-        """Update of PO file doesn't corrupt it with non-UTF-8 encoding on Python3+Windows (#23271)"""
+        """
+        Update of PO file doesn't corrupt it with non-UTF-8 encoding on Windows
+        (#23271).
+        """
         BR_PO_BASE = 'locale/pt_BR/LC_MESSAGES/django'
-        os.chdir(self.test_dir)
         shutil.copyfile(BR_PO_BASE + '.pristine', BR_PO_BASE + '.po')
-        self.addCleanup(self.rmfile, os.path.join(self.test_dir, 'locale', 'pt_BR', 'LC_MESSAGES', 'django.po'))
         management.call_command('makemessages', locale=['pt_BR'], verbosity=0)
         self.assertTrue(os.path.exists(BR_PO_BASE + '.po'))
-        with io.open(BR_PO_BASE + '.po', 'r', encoding='utf-8') as fp:
-            po_contents = force_text(fp.read())
+        with open(BR_PO_BASE + '.po', encoding='utf-8') as fp:
+            po_contents = fp.read()
             self.assertMsgStr("Größe", po_contents)
 
+    def test_pot_charset_header_is_utf8(self):
+        """Content-Type: ... charset=CHARSET is replaced with charset=UTF-8"""
+        msgs = (
+            '# SOME DESCRIPTIVE TITLE.\n'
+            '# (some lines truncated as they are not relevant)\n'
+            '"Content-Type: text/plain; charset=CHARSET\\n"\n'
+            '"Content-Transfer-Encoding: 8bit\\n"\n'
+            '\n'
+            '#: somefile.py:8\n'
+            'msgid "mañana; charset=CHARSET"\n'
+            'msgstr ""\n'
+        )
+        with tempfile.NamedTemporaryFile() as pot_file:
+            pot_filename = pot_file.name
+        write_pot_file(pot_filename, msgs)
+        with open(pot_filename, encoding='utf-8') as fp:
+            pot_contents = fp.read()
+            self.assertIn('Content-Type: text/plain; charset=UTF-8', pot_contents)
+            self.assertIn('mañana; charset=CHARSET', pot_contents)
+
 
 class JavascriptExtractorTests(ExtractorTests):
 
     PO_FILE = 'locale/%s/LC_MESSAGES/djangojs.po' % LOCALE
 
     def test_javascript_literals(self):
-        os.chdir(self.test_dir)
         _, po_contents = self._run_makemessages(domain='djangojs')
         self.assertMsgId('This literal should be included.', po_contents)
         self.assertMsgId('gettext_noop should, too.', po_contents)
@@ -450,16 +436,15 @@ class JavascriptExtractorTests(ExtractorTests):
         self.assertMsgId("quz", po_contents)
         self.assertMsgId("foobar", po_contents)
 
-    @override_settings(
-        STATIC_ROOT=os.path.join(this_directory, 'commands', 'static/'),
-        MEDIA_ROOT=os.path.join(this_directory, 'commands', 'media_root/'))
     def test_media_static_dirs_ignored(self):
         """
         Regression test for #23583.
         """
-        _, po_contents = self._run_makemessages(domain='djangojs')
-        self.assertMsgId("Static content inside app should be included.", po_contents)
-        self.assertNotMsgId("Content from STATIC_ROOT should not be included", po_contents)
+        with override_settings(STATIC_ROOT=os.path.join(self.test_dir, 'static/'),
+                               MEDIA_ROOT=os.path.join(self.test_dir, 'media_root/')):
+            _, po_contents = self._run_makemessages(domain='djangojs')
+            self.assertMsgId("Static content inside app should be included.", po_contents)
+            self.assertNotMsgId("Content from STATIC_ROOT should not be included", po_contents)
 
     @override_settings(STATIC_ROOT=None, MEDIA_ROOT='')
     def test_default_root_settings(self):
@@ -495,81 +480,49 @@ class IgnoredExtractorTests(ExtractorTests):
         self.assertIn("ignoring file xxx_ignored.html", out)
         self.assertNotMsgId('This should be ignored too.', po_contents)
 
-    @override_settings(
-        STATIC_ROOT=os.path.join(this_directory, 'commands', 'static/'),
-        MEDIA_ROOT=os.path.join(this_directory, 'commands', 'media_root/'))
     def test_media_static_dirs_ignored(self):
-        out, _ = self._run_makemessages()
-        self.assertIn("ignoring directory static", out)
-        self.assertIn("ignoring directory media_root", out)
+        with override_settings(STATIC_ROOT=os.path.join(self.test_dir, 'static/'),
+                               MEDIA_ROOT=os.path.join(self.test_dir, 'media_root/')):
+            out, _ = self._run_makemessages()
+            self.assertIn("ignoring directory static", out)
+            self.assertIn("ignoring directory media_root", out)
 
 
 class SymlinkExtractorTests(ExtractorTests):
 
     def setUp(self):
-        super(SymlinkExtractorTests, self).setUp()
+        super().setUp()
         self.symlinked_dir = os.path.join(self.test_dir, 'templates_symlinked')
 
-    def tearDown(self):
-        super(SymlinkExtractorTests, self).tearDown()
-        os.chdir(self.test_dir)
-        try:
-            os.remove(self.symlinked_dir)
-        except OSError:
-            pass
-        os.chdir(self._cwd)
-
     def test_symlink(self):
-        # On Python < 3.2 os.symlink() exists only on Unix
-        if hasattr(os, 'symlink'):
-            if os.path.exists(self.symlinked_dir):
-                self.assertTrue(os.path.islink(self.symlinked_dir))
-            else:
-                # On Python >= 3.2) os.symlink() exists always but then can
-                # fail at runtime when user hasn't the needed permissions on
-                # Windows versions that support symbolink links (>= 6/Vista).
-                # See Python issue 9333 (http://bugs.python.org/issue9333).
-                # Skip the test in that case
-                try:
-                    os.symlink(os.path.join(self.test_dir, 'templates'), self.symlinked_dir)
-                except (OSError, NotImplementedError):
-                    raise SkipTest("os.symlink() is available on this OS but can't be used by this user.")
-            os.chdir(self.test_dir)
-            management.call_command('makemessages', locale=[LOCALE], verbosity=0, symlinks=True)
-            self.assertTrue(os.path.exists(self.PO_FILE))
-            with open(self.PO_FILE, 'r') as fp:
-                po_contents = force_text(fp.read())
-                self.assertMsgId('This literal should be included.', po_contents)
-                self.assertIn('templates_symlinked/test.html', po_contents)
+        if symlinks_supported():
+            os.symlink(os.path.join(self.test_dir, 'templates'), self.symlinked_dir)
+        else:
+            self.skipTest("os.symlink() not available on this OS + Python version combination.")
+        management.call_command('makemessages', locale=[LOCALE], verbosity=0, symlinks=True)
+        self.assertTrue(os.path.exists(self.PO_FILE))
+        with open(self.PO_FILE) as fp:
+            po_contents = fp.read()
+            self.assertMsgId('This literal should be included.', po_contents)
+        self.assertLocationCommentPresent(self.PO_FILE, None, 'templates_symlinked', 'test.html')
 
 
 class CopyPluralFormsExtractorTests(ExtractorTests):
 
     PO_FILE_ES = 'locale/es/LC_MESSAGES/django.po'
 
-    def tearDown(self):
-        super(CopyPluralFormsExtractorTests, self).tearDown()
-        os.chdir(self.test_dir)
-        try:
-            self._rmrf('locale/es')
-        except OSError:
-            pass
-        os.chdir(self._cwd)
-
     def test_copy_plural_forms(self):
-        os.chdir(self.test_dir)
         management.call_command('makemessages', locale=[LOCALE], verbosity=0)
         self.assertTrue(os.path.exists(self.PO_FILE))
-        with open(self.PO_FILE, 'r') as fp:
-            po_contents = force_text(fp.read())
+        with open(self.PO_FILE) as fp:
+            po_contents = fp.read()
             self.assertIn('Plural-Forms: nplurals=2; plural=(n != 1)', po_contents)
 
     def test_override_plural_forms(self):
         """Ticket #20311."""
-        os.chdir(self.test_dir)
         management.call_command('makemessages', locale=['es'], extensions=['djtpl'], verbosity=0)
         self.assertTrue(os.path.exists(self.PO_FILE_ES))
-        with io.open(self.PO_FILE_ES, 'r', encoding='utf-8') as fp:
+        with open(self.PO_FILE_ES, encoding='utf-8') as fp:
             po_contents = fp.read()
             found = re.findall(r'^(?P<value>"Plural-Forms.+?\\n")\s*$', po_contents, re.MULTILINE | re.DOTALL)
             self.assertEqual(1, len(found))
@@ -580,11 +533,10 @@ class CopyPluralFormsExtractorTests(ExtractorTests):
         found inside a {% trans %} tag and also in another file inside a
         {% blocktrans %} with a plural (#17375).
         """
-        os.chdir(self.test_dir)
         management.call_command('makemessages', locale=[LOCALE], extensions=['html', 'djtpl'], verbosity=0)
         self.assertTrue(os.path.exists(self.PO_FILE))
-        with open(self.PO_FILE, 'r') as fp:
-            po_contents = force_text(fp.read())
+        with open(self.PO_FILE) as fp:
+            po_contents = fp.read()
             self.assertNotIn("#-#-#-#-#  django.pot (PACKAGE VERSION)  #-#-#-#-#\\n", po_contents)
             self.assertMsgId('First `trans`, then `blocktrans` with a plural', po_contents)
             self.assertMsgIdPlural('Plural for a `trans` and `blocktrans` collision case', po_contents)
@@ -593,11 +545,10 @@ class CopyPluralFormsExtractorTests(ExtractorTests):
 class NoWrapExtractorTests(ExtractorTests):
 
     def test_no_wrap_enabled(self):
-        os.chdir(self.test_dir)
         management.call_command('makemessages', locale=[LOCALE], verbosity=0, no_wrap=True)
         self.assertTrue(os.path.exists(self.PO_FILE))
-        with open(self.PO_FILE, 'r') as fp:
-            po_contents = force_text(fp.read())
+        with open(self.PO_FILE) as fp:
+            po_contents = fp.read()
             self.assertMsgId(
                 'This literal should also be included wrapped or not wrapped '
                 'depending on the use of the --no-wrap option.',
@@ -605,11 +556,10 @@ class NoWrapExtractorTests(ExtractorTests):
             )
 
     def test_no_wrap_disabled(self):
-        os.chdir(self.test_dir)
         management.call_command('makemessages', locale=[LOCALE], verbosity=0, no_wrap=False)
         self.assertTrue(os.path.exists(self.PO_FILE))
-        with open(self.PO_FILE, 'r') as fp:
-            po_contents = force_text(fp.read())
+        with open(self.PO_FILE) as fp:
+            po_contents = fp.read()
             self.assertMsgId(
                 '""\n"This literal should also be included wrapped or not '
                 'wrapped depending on the "\n"use of the --no-wrap option."',
@@ -622,83 +572,114 @@ class LocationCommentsTests(ExtractorTests):
 
     def test_no_location_enabled(self):
         """Behavior is correct if --no-location switch is specified. See #16903."""
-        os.chdir(self.test_dir)
         management.call_command('makemessages', locale=[LOCALE], verbosity=0, no_location=True)
         self.assertTrue(os.path.exists(self.PO_FILE))
-        self.assertLocationCommentNotPresent(self.PO_FILE, 55, 'templates', 'test.html.py')
+        self.assertLocationCommentNotPresent(self.PO_FILE, None, 'test.html')
 
     def test_no_location_disabled(self):
         """Behavior is correct if --no-location switch isn't specified."""
-        os.chdir(self.test_dir)
         management.call_command('makemessages', locale=[LOCALE], verbosity=0, no_location=False)
         self.assertTrue(os.path.exists(self.PO_FILE))
         # #16903 -- Standard comment with source file relative path should be present
         self.assertLocationCommentPresent(self.PO_FILE, 'Translatable literal #6b', 'templates', 'test.html')
 
-        # #21208 -- Leaky paths in comments on Windows e.g. #: path\to\file.html.py:123
-        self.assertLocationCommentNotPresent(self.PO_FILE, None, 'templates', 'test.html.py')
+    def test_location_comments_for_templatized_files(self):
+        """
+        Ensure no leaky paths in comments, e.g. #: path\to\file.html.py:123
+        Refs #21209/#26341.
+        """
+        management.call_command('makemessages', locale=[LOCALE], verbosity=0)
+        self.assertTrue(os.path.exists(self.PO_FILE))
+        with open(self.PO_FILE) as fp:
+            po_contents = fp.read()
+        self.assertMsgId('#: templates/test.html.py', po_contents)
+        self.assertLocationCommentNotPresent(self.PO_FILE, None, '.html.py')
+        self.assertLocationCommentPresent(self.PO_FILE, 5, 'templates', 'test.html')
+
+    @requires_gettext_019
+    def test_add_location_full(self):
+        """makemessages --add-location=full"""
+        management.call_command('makemessages', locale=[LOCALE], verbosity=0, add_location='full')
+        self.assertTrue(os.path.exists(self.PO_FILE))
+        # Comment with source file relative path and line number is present.
+        self.assertLocationCommentPresent(self.PO_FILE, 'Translatable literal #6b', 'templates', 'test.html')
+
+    @requires_gettext_019
+    def test_add_location_file(self):
+        """makemessages --add-location=file"""
+        management.call_command('makemessages', locale=[LOCALE], verbosity=0, add_location='file')
+        self.assertTrue(os.path.exists(self.PO_FILE))
+        # Comment with source file relative path is present.
+        self.assertLocationCommentPresent(self.PO_FILE, None, 'templates', 'test.html')
+        # But it should not contain the line number.
+        self.assertLocationCommentNotPresent(self.PO_FILE, 'Translatable literal #6b', 'templates', 'test.html')
+
+    @requires_gettext_019
+    def test_add_location_never(self):
+        """makemessages --add-location=never"""
+        management.call_command('makemessages', locale=[LOCALE], verbosity=0, add_location='never')
+        self.assertTrue(os.path.exists(self.PO_FILE))
+        self.assertLocationCommentNotPresent(self.PO_FILE, None, 'test.html')
+
+    @mock.patch('django.core.management.commands.makemessages.Command.gettext_version', new=(0, 18, 99))
+    def test_add_location_gettext_version_check(self):
+        """
+        CommandError is raised when using makemessages --add-location with
+        gettext < 0.19.
+        """
+        msg = "The --add-location option requires gettext 0.19 or later. You have 0.18.99."
+        with self.assertRaisesMessage(CommandError, msg):
+            management.call_command('makemessages', locale=[LOCALE], verbosity=0, add_location='full')
 
 
 class KeepPotFileExtractorTests(ExtractorTests):
 
     POT_FILE = 'locale/django.pot'
 
-    def tearDown(self):
-        super(KeepPotFileExtractorTests, self).tearDown()
-        os.chdir(self.test_dir)
-        try:
-            os.unlink(self.POT_FILE)
-        except OSError:
-            pass
-        os.chdir(self._cwd)
-
     def test_keep_pot_disabled_by_default(self):
-        os.chdir(self.test_dir)
         management.call_command('makemessages', locale=[LOCALE], verbosity=0)
         self.assertFalse(os.path.exists(self.POT_FILE))
 
     def test_keep_pot_explicitly_disabled(self):
-        os.chdir(self.test_dir)
-        management.call_command('makemessages', locale=[LOCALE], verbosity=0,
-                                keep_pot=False)
+        management.call_command('makemessages', locale=[LOCALE], verbosity=0, keep_pot=False)
         self.assertFalse(os.path.exists(self.POT_FILE))
 
     def test_keep_pot_enabled(self):
-        os.chdir(self.test_dir)
-        management.call_command('makemessages', locale=[LOCALE], verbosity=0,
-                                keep_pot=True)
+        management.call_command('makemessages', locale=[LOCALE], verbosity=0, keep_pot=True)
         self.assertTrue(os.path.exists(self.POT_FILE))
 
 
 class MultipleLocaleExtractionTests(ExtractorTests):
     PO_FILE_PT = 'locale/pt/LC_MESSAGES/django.po'
     PO_FILE_DE = 'locale/de/LC_MESSAGES/django.po'
+    PO_FILE_KO = 'locale/ko/LC_MESSAGES/django.po'
     LOCALES = ['pt', 'de', 'ch']
 
-    def tearDown(self):
-        super(MultipleLocaleExtractionTests, self).tearDown()
-        os.chdir(self.test_dir)
-        for locale in self.LOCALES:
-            try:
-                self._rmrf('locale/%s' % locale)
-            except OSError:
-                pass
-        os.chdir(self._cwd)
-
     def test_multiple_locales(self):
-        os.chdir(self.test_dir)
         management.call_command('makemessages', locale=['pt', 'de'], verbosity=0)
         self.assertTrue(os.path.exists(self.PO_FILE_PT))
         self.assertTrue(os.path.exists(self.PO_FILE_DE))
 
+    def test_all_locales(self):
+        """
+        When the `locale` flag is absent, all dirs from the parent locale dir
+        are considered as language directories, except if the directory doesn't
+        start with two letters (which excludes __pycache__, .gitignore, etc.).
+        """
+        os.mkdir(os.path.join('locale', '_do_not_pick'))
+        # Excluding locales that do not compile
+        management.call_command('makemessages', exclude=['ja', 'es_AR'], verbosity=0)
+        self.assertTrue(os.path.exists(self.PO_FILE_KO))
+        self.assertFalse(os.path.exists('locale/_do_not_pick/LC_MESSAGES/django.po'))
+
 
 class ExcludedLocaleExtractionTests(ExtractorTests):
 
+    work_subdir = 'exclude'
+
     LOCALES = ['en', 'fr', 'it']
     PO_FILE = 'locale/%s/LC_MESSAGES/django.po'
 
-    test_dir = os.path.abspath(os.path.join(this_directory, 'exclude'))
-
     def _set_times_for_all_po_files(self):
         """
         Set access and modification times to the Unix epoch time for all the .po files.
@@ -707,11 +688,9 @@ class ExcludedLocaleExtractionTests(ExtractorTests):
             os.utime(self.PO_FILE % locale, (0, 0))
 
     def setUp(self):
-        super(ExcludedLocaleExtractionTests, self).setUp()
-        os.chdir(self.test_dir)  # ExtractorTests.tearDown() takes care of restoring.
-        shutil.copytree('canned_locale', 'locale')
+        super().setUp()
+        copytree('canned_locale', 'locale')
         self._set_times_for_all_po_files()
-        self.addCleanup(self._rmrf, os.path.join(self.test_dir, 'locale'))
 
     def test_command_help(self):
         with captured_stdout(), captured_stderr():
@@ -748,43 +727,39 @@ class ExcludedLocaleExtractionTests(ExtractorTests):
 
 class CustomLayoutExtractionTests(ExtractorTests):
 
-    def setUp(self):
-        super(CustomLayoutExtractionTests, self).setUp()
-        self.test_dir = os.path.join(this_directory, 'project_dir')
+    work_subdir = 'project_dir'
 
     def test_no_locale_raises(self):
-        os.chdir(self.test_dir)
-        with six.assertRaisesRegex(self, management.CommandError,
-                "Unable to find a locale path to store translations for file"):
+        msg = "Unable to find a locale path to store translations for file"
+        with self.assertRaisesMessage(management.CommandError, msg):
             management.call_command('makemessages', locale=LOCALE, verbosity=0)
 
-    @override_settings(
-        LOCALE_PATHS=[os.path.join(this_directory, 'project_dir', 'project_locale')],
-    )
     def test_project_locale_paths(self):
         """
-        Test that:
-          * translations for an app containing a locale folder are stored in that folder
-          * translations outside of that app are in LOCALE_PATHS[0]
+        * translations for an app containing a locale folder are stored in that folder
+        * translations outside of that app are in LOCALE_PATHS[0]
         """
-        os.chdir(self.test_dir)
-        self.addCleanup(shutil.rmtree,
-            os.path.join(settings.LOCALE_PATHS[0], LOCALE), True)
-        self.addCleanup(shutil.rmtree,
-            os.path.join(self.test_dir, 'app_with_locale', 'locale', LOCALE), True)
+        with override_settings(LOCALE_PATHS=[os.path.join(self.test_dir, 'project_locale')]):
+            management.call_command('makemessages', locale=[LOCALE], verbosity=0)
+            project_de_locale = os.path.join(
+                self.test_dir, 'project_locale', 'de', 'LC_MESSAGES', 'django.po')
+            app_de_locale = os.path.join(
+                self.test_dir, 'app_with_locale', 'locale', 'de', 'LC_MESSAGES', 'django.po')
+            self.assertTrue(os.path.exists(project_de_locale))
+            self.assertTrue(os.path.exists(app_de_locale))
 
-        management.call_command('makemessages', locale=[LOCALE], verbosity=0)
-        project_de_locale = os.path.join(
-            self.test_dir, 'project_locale', 'de', 'LC_MESSAGES', 'django.po')
-        app_de_locale = os.path.join(
-            self.test_dir, 'app_with_locale', 'locale', 'de', 'LC_MESSAGES', 'django.po')
-        self.assertTrue(os.path.exists(project_de_locale))
-        self.assertTrue(os.path.exists(app_de_locale))
+            with open(project_de_locale) as fp:
+                po_contents = fp.read()
+                self.assertMsgId('This app has no locale directory', po_contents)
+                self.assertMsgId('This is a project-level string', po_contents)
+            with open(app_de_locale) as fp:
+                po_contents = fp.read()
+                self.assertMsgId('This app has a locale directory', po_contents)
 
-        with open(project_de_locale, 'r') as fp:
-            po_contents = force_text(fp.read())
-            self.assertMsgId('This app has no locale directory', po_contents)
-            self.assertMsgId('This is a project-level string', po_contents)
-        with open(app_de_locale, 'r') as fp:
-            po_contents = force_text(fp.read())
-            self.assertMsgId('This app has a locale directory', po_contents)
+
+@skipUnless(has_xgettext, 'xgettext is mandatory for extraction tests')
+class NoSettingsExtractionTests(AdminScriptTestCase):
+    def test_makemessages_no_settings(self):
+        out, err = self.run_django_admin(['makemessages', '-l', 'en', '-v', '0'])
+        self.assertNoOutput(err)
+        self.assertNoOutput(out)
